import type { Position } from 'vscode-languageserver/node';
import type { Range } from 'vscode-languageserver/node';
import type { TextDocument } from 'vscode-languageserver-textdocument';
import type { CssSourceMap, TeleportMappingData } from '../utils/sourceMaps';
import type { TeleportSideData } from '../utils/sourceMaps';
import type { TsMappingData } from '../utils/sourceMaps';
import type { TsSourceMap } from '../utils/sourceMaps';
import type { LanguageService as TsLanguageService } from 'vscode-typescript-languageservice';
import type { LanguageService as CssLanguageService } from 'vscode-css-languageservice';
import type { LanguageService as HtmlLanguageService } from 'vscode-html-languageservice';
import type { LanguageService as PugLanguageService } from 'vscode-pug-languageservice';
import type { Stylesheet } from 'vscode-css-languageservice';
import type { HTMLDocument } from 'vscode-html-languageservice';
import type { PugDocument } from 'vscode-pug-languageservice';
import type { SourceFile } from '../sourceFile';
import type * as ts from 'typescript';
import { Range as MapedRange } from '@volar/source-map';
export declare function createMapper(sourceFiles: Map<string, SourceFile>, tsLanguageService: TsLanguageService, getTextDocument: (uri: string) => TextDocument | undefined): {
    css: {
        from: (cssUri: string, cssStart: Position, cssEnd?: Position | undefined) => {
            sourceMap: CssSourceMap;
            textDocument: TextDocument;
            range: Range;
        }[];
        to: (vueUri: string, vueStart: Position, vueEnd?: Position | undefined) => {
            sourceMap: CssSourceMap;
            textDocument: TextDocument;
            stylesheet: Stylesheet;
            range: Range;
            languageService: CssLanguageService;
        }[];
    };
    html: {
        from: (htmlUri: string, htmlStart: Position, htmlEnd?: Position | undefined) => {
            textDocument: TextDocument;
            range: Range;
        }[];
        to: (vueUri: string, vueStart: Position, vueEnd?: Position | undefined) => ({
            language: 'html';
            textDocument: TextDocument;
            htmlDocument: HTMLDocument;
            range: Range;
            languageService: HtmlLanguageService;
        } | {
            language: 'pug';
            textDocument: TextDocument;
            pugDocument: PugDocument;
            range: Range;
            languageService: PugLanguageService;
        })[];
    };
    tsUri: {
        from: (tsUri: string) => TextDocument | undefined;
        to: (vueUri: string) => {
            languageService: {
                findDefinition: (uri: string, position: Position) => import("vscode-languageserver/node").LocationLink[];
                findTypeDefinition: (uri: string, position: Position) => import("vscode-languageserver/node").LocationLink[];
                findReferences: (uri: string, position: Position) => import("vscode-languageserver/node").Location[];
                prepareRename: (uri: string, position: Position) => Range | import("vscode-languageserver/node").ResponseError<void> | undefined;
                doRename: (uri: string, position: Position, newName: string) => import("vscode-languageserver/node").WorkspaceEdit | undefined;
                getEditsForFileRename: (oldUri: string, newUri: string) => import("vscode-languageserver/node").WorkspaceEdit | undefined;
                getCodeActions: (uri: string, range: Range, context: import("vscode-languageserver/node").CodeActionContext) => import("vscode-languageserver/node").CodeAction[] | undefined;
                doCodeActionResolve: (codeAction: import("vscode-languageserver/node").CodeAction) => import("vscode-languageserver/node").CodeAction;
                findDocumentHighlights: (uri: string, position: Position) => import("vscode-languageserver/node").DocumentHighlight[];
                findDocumentSymbols: (uri: string) => import("vscode-languageserver/node").DocumentSymbol[];
                findWorkspaceSymbols: (uri: string) => import("vscode-languageserver/node").SymbolInformation[];
                doComplete: (uri: string, position: Position, options?: ts.GetCompletionsAtPositionOptions | undefined) => import("vscode-languageserver/node").CompletionItem[];
                doCompletionResolve: (item: import("vscode-languageserver/node").CompletionItem, newOffset?: number | undefined) => import("vscode-languageserver/node").CompletionItem;
                doHover: (uri: string, position: Position, documentOnly?: boolean | undefined) => import("vscode-languageserver/node").Hover | undefined;
                doFormatting: (uri: string, options: import("vscode-languageserver/node").FormattingOptions, range?: Range | undefined) => import("vscode-languageserver/node").TextEdit[];
                getSignatureHelp: (uri: string, position: Position) => import("vscode-languageserver/node").SignatureHelp | undefined;
                getSelectionRange: (uri: string, position: Position) => import("vscode-languageserver/node").SelectionRange | undefined;
                doValidation: (uri: string, options: {
                    semantic?: boolean | undefined;
                    syntactic?: boolean | undefined;
                    suggestion?: boolean | undefined;
                    declaration?: boolean | undefined;
                }, cancellationToken?: ts.CancellationToken | undefined) => import("vscode-languageserver/node").Diagnostic[];
                getFoldingRanges: (uri: string) => import("vscode-languageserver/node").FoldingRange[];
                getDocumentSemanticTokens: (uri: string, range: Range, cancle?: import("vscode-languageserver/node").CancellationToken | undefined) => [number, number, number, number, number][] | undefined;
                callHierarchy: {
                    doPrepare: (uri: string, position: Position) => import("vscode-languageserver/node").CallHierarchyItem[];
                    getIncomingCalls: (item: import("vscode-languageserver/node").CallHierarchyItem) => import("vscode-languageserver/node").CallHierarchyIncomingCall[];
                    getOutgoingCalls: (item: import("vscode-languageserver/node").CallHierarchyItem) => import("vscode-languageserver/node").CallHierarchyOutgoingCall[];
                };
                dispose: () => void;
                __internal__: {
                    raw: import("typescript/lib/tsserverlibrary").LanguageService;
                    host: {
                        getImportSuggestionsCache: () => any;
                        getCompilationSettings(): ts.CompilerOptions;
                        getNewLine?(): string;
                        getProjectVersion?(): string;
                        getScriptFileNames(): string[];
                        getScriptKind?(fileName: string): ts.ScriptKind;
                        getScriptVersion(fileName: string): string;
                        getScriptSnapshot(fileName: string): ts.IScriptSnapshot | undefined;
                        getProjectReferences?(): readonly ts.ProjectReference[] | undefined;
                        getLocalizedDiagnosticMessages?(): any;
                        getCancellationToken?(): ts.HostCancellationToken;
                        getCurrentDirectory(): string;
                        getDefaultLibFileName(options: ts.CompilerOptions): string;
                        log?(s: string): void;
                        trace?(s: string): void;
                        error?(s: string): void;
                        useCaseSensitiveFileNames?(): boolean;
                        readDirectory?(path: string, extensions?: readonly string[] | undefined, exclude?: readonly string[] | undefined, include?: readonly string[] | undefined, depth?: number | undefined): string[];
                        readFile?(path: string, encoding?: string | undefined): string | undefined;
                        realpath?(path: string): string;
                        fileExists?(path: string): boolean;
                        getTypeRootsVersion?(): number;
                        resolveModuleNames?(moduleNames: string[], containingFile: string, reusedNames: string[] | undefined, redirectedReference: ts.ResolvedProjectReference | undefined, options: ts.CompilerOptions): (ts.ResolvedModule | undefined)[];
                        getResolvedModuleWithFailedLookupLocationsFromCache?(modulename: string, containingFile: string): ts.ResolvedModuleWithFailedLookupLocations | undefined;
                        resolveTypeReferenceDirectives?(typeDirectiveNames: string[], containingFile: string, redirectedReference: ts.ResolvedProjectReference | undefined, options: ts.CompilerOptions): (ts.ResolvedTypeReferenceDirective | undefined)[];
                        getDirectories?(directoryName: string): string[];
                        getCustomTransformers?(): ts.CustomTransformers | undefined;
                        isKnownTypesPackageName?(name: string): boolean;
                        installPackage?(options: ts.InstallPackageOptions): Promise<ts.ApplyCodeActionCommandResult>;
                        writeFile?(fileName: string, content: string): void;
                        getParsedCommandLine?(fileName: string): ts.ParsedCommandLine | undefined;
                        directoryExists?(directoryName: string): boolean;
                    };
                    getTextDocument: (uri: string) => TextDocument | undefined;
                    getTextDocument2: (uri: string) => TextDocument | undefined;
                };
            };
            textDocument: TextDocument;
            isVirtualFile: boolean;
        } | undefined;
    };
    ts: {
        from: (tsUri: string, tsStart: Position, tsEnd?: Position | undefined) => {
            textDocument: TextDocument;
            range: Range;
            data?: TsMappingData | undefined;
        }[];
        from2: (tsFsPath: string, tsStart: number, tsEnd?: number | undefined) => {
            fileName: string;
            textDocument: TextDocument;
            range: MapedRange;
            data?: TsMappingData | undefined;
        }[];
        to: (vueUri: string, vueStart: Position, vueEnd?: Position | undefined) => {
            sourceMap: TsSourceMap | undefined;
            textDocument: TextDocument;
            range: Range;
            data: TsMappingData;
            languageService: TsLanguageService;
        }[];
        to2: (vueFsPath: string, vueStart: number, vueEnd?: number | undefined) => {
            sourceMap: TsSourceMap | undefined;
            fileName: string;
            textDocument: TextDocument;
            range: MapedRange;
            data: TsMappingData;
            languageService: ts.LanguageService;
        }[];
        teleports: (tsUri: string, tsStart: Position, tsEnd?: Position | undefined) => {
            data: TeleportMappingData;
            sideData: TeleportSideData;
            start: Position;
            end: Position;
        }[];
        teleports2: (tsFsPath: string, tsStart: number, tsEnd?: number | undefined) => {
            data: TeleportMappingData;
            sideData: TeleportSideData;
            start: number;
            end: number;
        }[];
    };
    findSourceFileByTsUri: (tsUri: string) => {
        uri: string;
        getTemplateTagNames: () => Set<string> | undefined;
        getTemplateAttrNames: () => Set<string> | undefined;
        getTextDocument: () => {
            readonly uri: string;
            readonly languageId: string;
            readonly version: number;
            getText: (range?: import("vscode-languageserver-textdocument").Range | undefined) => string;
            positionAt: (offset: number) => import("vscode-languageserver-textdocument").Position;
            offsetAt: (position: import("vscode-languageserver-textdocument").Position) => number;
            readonly lineCount: number;
        };
        update: (newVueDocument: TextDocument) => {
            scriptUpdated: boolean;
            templateScriptUpdated: boolean;
        };
        updateTemplateScript: () => boolean;
        getComponentCompletionData: () => Map<string, {
            item: import("vscode-languageserver/node").CompletionItem | undefined;
            bind: import("vscode-languageserver/node").CompletionItem[];
            on: import("vscode-languageserver/node").CompletionItem[];
            slot: import("vscode-languageserver/node").CompletionItem[];
        }>;
        getDiagnostics: (response: (diags: import("vscode-languageserver/node").Diagnostic[]) => void, isCancel?: (() => Promise<boolean>) | undefined) => Promise<void>;
        getTsSourceMaps: () => TsSourceMap[];
        getMainTsDoc: () => TextDocument;
        getCssSourceMaps: () => CssSourceMap[];
        getJsonSourceMaps: () => import("../utils/sourceMaps").JsonSourceMap[];
        getHtmlSourceMaps: () => import("../utils/sourceMaps").HtmlSourceMap[];
        getPugSourceMaps: () => import("../utils/sourceMaps").PugSourceMap[];
        getTemplateScriptData: () => {
            projectVersion: string | undefined;
            context: string[];
            componentItems: {
                label: string;
                labelDetails?: {
                    parameters?: string | undefined;
                    qualifier?: string | undefined;
                    type?: string | undefined;
                } | undefined;
                kind?: import("vscode-languageserver/node").CompletionItemKind | undefined;
                tags?: 1[] | undefined;
                detail?: string | undefined;
                documentation?: string | {
                    kind: import("vscode-languageserver/node").MarkupKind;
                    value: string;
                } | undefined;
                deprecated?: boolean | undefined;
                preselect?: boolean | undefined;
                sortText?: string | undefined;
                filterText?: string | undefined;
                insertText?: string | undefined;
                insertTextFormat?: import("vscode-languageserver/node").InsertTextFormat | undefined;
                insertTextMode?: import("vscode-languageserver/node").InsertTextMode | undefined;
                textEdit?: {
                    range: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    newText: string;
                } | {
                    newText: string;
                    insert: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    replace: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                } | undefined;
                additionalTextEdits?: {
                    range: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    newText: string;
                }[] | undefined;
                commitCharacters?: string[] | undefined;
                command?: {
                    title: string;
                    command: string;
                    arguments?: any[] | undefined;
                } | undefined;
                data?: any;
            }[];
            components: string[];
            props: string[];
            setupReturns: string[];
            htmlElementItems: {
                label: string;
                labelDetails?: {
                    parameters?: string | undefined;
                    qualifier?: string | undefined;
                    type?: string | undefined;
                } | undefined;
                kind?: import("vscode-languageserver/node").CompletionItemKind | undefined;
                tags?: 1[] | undefined;
                detail?: string | undefined;
                documentation?: string | {
                    kind: import("vscode-languageserver/node").MarkupKind;
                    value: string;
                } | undefined;
                deprecated?: boolean | undefined;
                preselect?: boolean | undefined;
                sortText?: string | undefined;
                filterText?: string | undefined;
                insertText?: string | undefined;
                insertTextFormat?: import("vscode-languageserver/node").InsertTextFormat | undefined;
                insertTextMode?: import("vscode-languageserver/node").InsertTextMode | undefined;
                textEdit?: {
                    range: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    newText: string;
                } | {
                    newText: string;
                    insert: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    replace: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                } | undefined;
                additionalTextEdits?: {
                    range: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    newText: string;
                }[] | undefined;
                commitCharacters?: string[] | undefined;
                command?: {
                    title: string;
                    command: string;
                    arguments?: any[] | undefined;
                } | undefined;
                data?: any;
            }[];
            htmlElements: string[];
        };
        getTeleports: () => import("../utils/sourceMaps").TeleportSourceMap[];
        getDescriptor: () => {
            template: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
            } | null;
            script: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
                src?: string | undefined;
            } | null;
            scriptSetup: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
            } | null;
            styles: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
                module: boolean;
                scoped: boolean;
            }[];
            customBlocks: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
                type: string;
            }[];
        };
        getVueHtmlDocument: () => HTMLDocument;
        getTsDocuments: () => Map<string, TextDocument>;
        getVirtualScript: () => {
            document: TextDocument | undefined;
            sourceMap: TsSourceMap | undefined;
        };
        getScriptSetupData: () => {
            labels: {
                start: number;
                end: number;
                binarys: {
                    parent: {
                        start: number;
                        end: number;
                    };
                    vars: {
                        isShortand: boolean;
                        inRoot: boolean;
                        start: number;
                        end: number;
                    }[];
                    left: {
                        start: number;
                        end: number;
                    };
                    right: {
                        start: number;
                        end: number;
                        isComputedCall: boolean;
                        withoutAs: {
                            start: number;
                            end: number;
                        };
                        as: {
                            start: number;
                            end: number;
                        } | undefined;
                    } | undefined;
                }[];
                label: {
                    start: number;
                    end: number;
                };
                parent: {
                    start: number;
                    end: number;
                };
            }[];
            returnVarNames: {
                start: number;
                end: number;
            }[];
            defineProps: {
                start: number;
                end: number;
                args?: {
                    start: number;
                    end: number;
                } | undefined;
                typeArgs?: {
                    start: number;
                    end: number;
                } | undefined;
            } | undefined;
            defineEmit: {
                start: number;
                end: number;
                args?: {
                    start: number;
                    end: number;
                } | undefined;
                typeArgs?: {
                    start: number;
                    end: number;
                } | undefined;
            } | undefined;
            dollars: number[];
        } | undefined;
        getScriptsRaw: () => {
            documents: TextDocument[];
            sourceMaps: TsSourceMap[];
        };
        getTemplateScriptFormat: () => {
            document: TextDocument | undefined;
            sourceMap: TsSourceMap | undefined;
        };
        shouldVerifyTsScript: (tsUri: string, mode: 2 | 1 | 3 | 4) => "all" | "none" | "unused";
    } | undefined;
};
